@@ -56,7 +56,7 @@ al1REF <- c("",
             "", 
             "",
             "Discrete",
-            "                     Level st04q01     N      wtdN   Percent StandardError", 
+            "                cpro_Level st04q01     N      wtdN   Percent StandardError", 
             " Below Proficiency Level 1  Female  95.4  1541.697  3.539350     0.7086689", 
             "    At Proficiency Level 1  Female 234.2  3815.988  8.760555     0.8224039", 
             "    At Proficiency Level 2  Female 524.6  8669.991 19.904132     0.9013387", 
@@ -76,7 +76,7 @@ al1REF <- c("",
 pgap2REF <- c("gapList",
               "Call: gap(variable = \"math\", data = usaINT2012, groupA = st04q01 == ", 
               "    \"Male\", groupB = st04q01 == \"Female\", percentiles = c(50, ", 
-              "    90), weightVar = \"w_fstuwt\")",
+              "    90), weightVar = \"w_fstuwt\", pctMethod = \"symmetric\")",
               "",
               "Labels:",
               " group          definition", 
@@ -90,7 +90,7 @@ pgap2REF <- c("gapList",
               "Results:",
               " percentiles estimateA estimateAse estimateB estimateBse   diffAB     covAB diffABse diffABpValue    dofAB", 
               "          50  480.7124    4.075956  474.6266    4.498829 6.085824 11.494158 3.723515    0.1064350 73.66955", 
-              "          90  605.2124    3.949571  595.4169    8.645421 9.795504  7.849511 8.639641    0.2599145 89.42849")
+              "          90  605.1735    3.942558  595.3941    8.603019 9.779404  7.893203 8.588906    0.2579161 89.16957")
 
 pvREF <- c("There are 10 subject scale(s) or subscale(s) in this edsurvey.data.frame:", 
            "'math' subject scale or subscale with 5 plausible values (the default).", 
